import 'dart:typed_data';
import 'package:flutter/material.dart';
import 'package:screenshot/screenshot.dart';
import 'dart:io';
import 'package:share/share.dart';
import 'package:file_picker/file_picker.dart';

enum ButtonPlacement{
  top,
  bottom,
}
Directory _temporaryDir = Directory.systemTemp;
Directory _saveFolder = Directory(_temporaryDir.path + "/raw_ware_quick_share");

Future<void> _screenshotSaver({
  required ScreenshotController screenshotController,
})async{
  File saveFile = File(_saveFolder.path + "/screenshot.png");
  //Capture screenshot
  Uint8List bytes = (await screenshotController.capture())!;
  //Save bytes into a file
  await saveFile.create(recursive: true);
  await saveFile.writeAsBytes(bytes);
  await Share.shareFiles([saveFile.path]);
  //Delete the cached files
  await _saveFolder.delete(recursive: true);
}

//One Tap Screenshot
class QuickShareScreenshot extends StatelessWidget {
  QuickShareScreenshot({ 
    Key? key,
    required this.child,
    required this.shareButton,
    required this.buttonPlacement,
  }) : super(key: key);
  final Widget child;
  final Widget shareButton;
  final ButtonPlacement buttonPlacement;
  final ScreenshotController  screenshotController = ScreenshotController();

  @override
  Widget build(BuildContext context) {
    if(buttonPlacement == ButtonPlacement.top){
      return Column(
        children: [
          GestureDetector(
            onTap: ()async{
              //Take, save and share screenshot
              await _screenshotSaver(
                screenshotController: screenshotController,
              );
            },
            child: shareButton,
          ),
          Screenshot(
            controller: screenshotController,
            child: child,
          ),
        ],
      );
    }else{
      return Column(
        children: [
          Screenshot(
            controller: screenshotController,
            child: child,
          ), 
          GestureDetector(
            onTap: ()async{
              //Take, save and share screenshot
              await _screenshotSaver(
                screenshotController: screenshotController,
              );
            },
            child: shareButton,
          ),
        ],
      );
    }
  }
}
class QuickShareFiles extends StatelessWidget {
  const QuickShareFiles({ 
    Key? key,
    required this.shareButton,
    this.allowMultipleFilesToBeSelected = true,
    this.allowedExtensions,
  }) : super(key: key);
  ///A widget that will trigger the file picker and share functionality
  final Widget shareButton;
  ///Wether or not multiple files can be selected. false for single file selection
  final bool allowMultipleFilesToBeSelected;
  final List<String>? allowedExtensions;

  @override
  Widget build(BuildContext context) {
    return GestureDetector(
      onTap: ()async{
        //Share files using file_picker: ^4.2.7 library
        FilePickerResult? result = await FilePicker.platform.pickFiles(
          allowMultiple: allowMultipleFilesToBeSelected,
          allowedExtensions: allowedExtensions,
        );
        //If files have been picked, share them
        if(result != null){
          List<String> filesPaths = [];
          for(String? path in result.paths){
            if(path != null){
              filesPaths.add(path);
            }
          }
          Share.shareFiles(filesPaths);
        }
      },
      child: shareButton,
    );
  }
}